


#define MY_RADIO_NRF24

#include <SPI.h>

#include <MySensors.h>
#include "Dimmer.h"

#include <DallasTemperature.h>
#include <OneWire.h>
#define COMPARE_TEMP 1 // Send temperature only if changed? 1 = Yes 0 = No
#define ONE_WIRE_BUS 4 // Pin where dallase sensor is connected 
#define MAX_ATTACHED_DS18B20 5

#define MY_NODE_ID 7 
#define CHILD_ID_LIGHT_1 3
#define CHILD_ID_LIGHT_2 2

#define EPROM_LIGHT_STATE_1 1
#define EPROM_DIMMER_LEVEL_1 2
#define EPROM_LIGHT_STATE_2 1
#define EPROM_DIMMER_LEVEL_2 2

#define LIGHT_OFF 0
#define LIGHT_ON 1

#define SN "VENTILATOR"
#define SV "1.0"

unsigned long measurementInterval = 30000;
//unsigned long SLEEP_TIME = 30000; // Sleep time between reads (in milliseconds)
OneWire oneWire(ONE_WIRE_BUS); // Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
DallasTemperature sensors(&oneWire); // Pass the oneWire reference to Dallas Temperature. 
float lastTemperature[MAX_ATTACHED_DS18B20];
int numSensors=0;
bool receivedConfig = false;
bool metric = true;


int16_t LastLightState_1=LIGHT_OFF;
int16_t LastDimValue_1=100;
int16_t LastLightState_2=LIGHT_OFF;
int16_t LastDimValue_2=100;


Dimmer dimmer_1(5, DIMMER_RAMP, 1.5);
Dimmer dimmer_2(6, DIMMER_RAMP, 1.5);

MyMessage lightMsg_1(CHILD_ID_LIGHT_1, V_LIGHT);
MyMessage dimmerMsg_1(CHILD_ID_LIGHT_1, V_DIMMER);
MyMessage lightMsg_2(CHILD_ID_LIGHT_2, V_LIGHT);
MyMessage dimmerMsg_2(CHILD_ID_LIGHT_2, V_DIMMER);

MyMessage msg(3,V_TEMP);



void before()
{
  
    sensors.begin();
}

void setup()
{

    for(int i=0; i<MAX_ATTACHED_DS18B20; i++) {
    lastTemperature[i] = 0;  
   }
    sensors.setWaitForConversion(false);
   

   
   dimmer_1.begin();
   dimmer_2.begin();
  int LightState_1=loadState(EPROM_LIGHT_STATE_1);
  if (LightState_1<=1) {
    LastLightState_1=LightState_1;
    int DimValue_1=loadState(EPROM_DIMMER_LEVEL_1);
    if ((DimValue_1>0)&&(DimValue_1<=100)) {

      LastDimValue_1=DimValue_1;
    }
  }

    int LightState_2=loadState(EPROM_LIGHT_STATE_2);
  if (LightState_2<=1) {
    LastLightState_2=LightState_1;
    int DimValue_2=loadState(EPROM_DIMMER_LEVEL_2);
    if ((DimValue_2>0)&&(DimValue_2<=100)) {

      LastDimValue_2=DimValue_2;
    }
  }


  SetCurrentState2Hardware();

 
}



void presentation()
{
    sendSketchInfo(SN, SV);
    sendSketchInfo("Temperature Sensor", "1.1");

  
    numSensors = sensors.getDeviceCount();
   
  
    for (int i=0; i<numSensors && i<MAX_ATTACHED_DS18B20; i++) {   
     present(i, S_TEMP);
  }
  

  present(CHILD_ID_LIGHT_1, S_DIMMER );
  present(CHILD_ID_LIGHT_2, S_DIMMER );
}

void loop()
{

   // You should not change these variables:
   static boolean isMeasuring = true;                // Used to indicate when the time is right for a new measurement to be made.
   static boolean isCalculating = false;                     // Used to bridge the time that is needed to calculate the temperature values by the Dallas library.
   static unsigned long currentMillis = 0;                   // The millisecond clock in the main loop.
   static unsigned long previousMeasurementMillis = 0;       // Used to remember the time of the last temperature measurement.
   static int16_t conversionTime = 0;                        // Used to store the time needed to calculate the temperature from measurements.

   currentMillis = millis(); // The time since the sensor started, counted in milliseconds. This script tries to avoid using the Sleep function, so that it could at the same time be a MySensors repeater.

   // Let's measure the temperature
   if(isMeasuring == true && currentMillis - previousMeasurementMillis >= measurementInterval) { // If we're not calculating, and enough time has passed, we'll start again.
      isMeasuring = false; // We're measuring, so let's take it off our to-do list.
      Serial.print("Starting new measurement(s)\n");
      previousMeasurementMillis = currentMillis; // Mark the time of the initialiation of this measurement.

      // Fetch temperatures from Dallas sensors
      sensors.requestTemperatures();

      // query conversion time. Apparently it takes a while to calculate.
      //CONVERSION_TIME = sensors.millisToWaitForConversion(sensors.getResolution());
      conversionTime = millisToWaitForConversion(sensors.getResolution());
      isCalculating = true; //Next step is to re-calculate the temperature again.
   }


   // Next, let's calculate and send the temperature
   if(isCalculating == true && currentMillis > previousMeasurementMillis + conversionTime ) {
      isCalculating = false; // check calculating off the to-do list too.

      for (int i=0; i<numSensors && i<MAX_ATTACHED_DS18B20; i++) {    // Loop through all the attached temperatur sensors.
         float temperature = static_cast<float>(static_cast<int>((getControllerConfig().isMetric?sensors.getTempCByIndex(i):sensors.getTempFByIndex(i)) * 10.)) / 10.;   // Fetch and round temperature to one decimal
         Serial.print("Sensor #");
         Serial.print(i);
         Serial.print(" says it is ");
         Serial.print(temperature);
         Serial.print(" degrees\n");
         if(temperature != -127.00 && temperature != 85.00) { // avoid working with measurement errors.
            if (COMPARE_TEMP == 1 && lastTemperature[i] == temperature) {
               Serial.print("Not sending it though, because it's the same temperature as before.\n");
            } else {
               Serial.print("Sending the temperature to the gateway.\n");
               send(msg.setSensor(i).set(temperature,1));
               lastTemperature[i] = temperature; // Save new temperatures to be able to compare in the next round.
            }
         }
      }

      // Both tasks are done. Time to wait until we should measure again.
      Serial.print("zzzzZZZZzzzzZZZZzzzz\n");

#ifdef BATTERY_POWERED
      unsigned long quicktimecheck = millis(); // check how much time has passed during the measurement (can be up to 750 milliseconds), and then calculate from that how long to sleep until the next intended measuring time.
      unsigned long sleeptime = measurementSleepTime - (quicktimecheck - previousMeasurementMillis); //How much time has passed already during the calculating? Subtract that from the intended interval time.
      sleep (sleeptime);
#endif

      isMeasuring = true;
   }
}



void receive(const MyMessage &message)
{
  if (message.sensor == CHILD_ID_LIGHT_1){
  if (message.type == V_LIGHT) {
    Serial.println( "V_LIGHT command received..." );

    int lstate_1= atoi( message.data );
    dimmer_1.set(lstate_1);
    LastLightState_1=lstate_1;
    saveState(EPROM_LIGHT_STATE_1, LastLightState_1);

    if ((LastLightState_1==LIGHT_ON)&&(LastDimValue_1==0)) {

      LastDimValue_1=100;
      saveState(EPROM_DIMMER_LEVEL_1, LastDimValue_1);
    }


  } else if (message.type == V_DIMMER) {
    
    int dimvalue_1= atoi( message.data );
    dimmer_1.set(dimvalue_1);
    if (dimvalue_1==0) {
      LastLightState_1=LIGHT_OFF;
    } else {
      LastLightState_1=LIGHT_ON;
      LastDimValue_1=dimvalue_1;
      saveState(EPROM_DIMMER_LEVEL_1, LastDimValue_1);
    }
   } 
  }
 if (message.sensor == CHILD_ID_LIGHT_2){
 if (message.type == V_LIGHT) {
    

    int lstate_2= atoi( message.data );
    dimmer_2.set(lstate_2);
    LastLightState_2=lstate_2;
    saveState(EPROM_LIGHT_STATE_2, LastLightState_2);

    if ((LastLightState_2==LIGHT_ON)&&(LastDimValue_2==0)) {
      LastDimValue_2=100;
      saveState(EPROM_DIMMER_LEVEL_2, LastDimValue_2);
    }


  } else if (message.type == V_DIMMER) {
    
    int dimvalue_2= atoi( message.data );
    dimmer_2.set(dimvalue_2);
    if (dimvalue_2==0) {
      LastLightState_2=LIGHT_OFF;
    } else {
      LastLightState_2=LIGHT_ON;
      LastDimValue_2=dimvalue_2;
      saveState(EPROM_DIMMER_LEVEL_2, LastDimValue_2);
    }
  } else {
    Serial.println( "Invalid command received..." );
    return;
  }
 }

  SetCurrentState2Hardware();
}

void SetCurrentState2Hardware()
{
  if (LastLightState_1==LIGHT_OFF) {
    Serial.println( "Light state_1: OFF" );
  } else {
    Serial.print( "Light state_1: ON, Level: " );
    Serial.println( LastDimValue_1 );
  }
    if (LastLightState_2==LIGHT_OFF) {
    Serial.println( "Light state_2: OFF" );
  } else {
    Serial.print( "Light state_2: ON, Level: " );
    Serial.println( LastDimValue_2 );
  }


  SendCurrentState2Controller();
}

void SendCurrentState2Controller()
{
  if ((LastLightState_1==LIGHT_OFF)||(LastDimValue_1==0)) {
    send(dimmerMsg_1.set((int16_t)0));
  } else {
    send(dimmerMsg_1.set(LastDimValue_1));
  }
    if ((LastLightState_2==LIGHT_OFF)||(LastDimValue_2==0)) {
    send(dimmerMsg_2.set((int16_t)0));
  } else {
    send(dimmerMsg_2.set(LastDimValue_2));
  }
}
int16_t millisToWaitForConversion(uint8_t bitResolution)
{
   switch (bitResolution) {
   case 9:
      return 94;
   case 10:
      return 188;
   case 11:
      return 375;
   default:
      return 750;
   }
}
